#ifndef __STM8S_FLASH_H
#define __STM8S_FLASH_H

//----------------------------------------------------------//
//                                        //
//----------------------------------------------------------//

//----------------------------------------------------------//
//                            //
//----------------------------------------------------------//

#define FLASH_CLEAR_BYTE    0x00
#define FLASH_CR2_WPRG      0x40
#define FLASH_NCR2_NWPRG    0x40

//----------------------------------------------------------//
//                                    //
//----------------------------------------------------------//

// FLASH programming modes
typedef enum {
    FLASH_PROGRAMMODE_STANDARD = (uint8_t)0x00,
    FLASH_PROGRAMMODE_FAST     = (uint8_t)0x10
} FLASH_ProgramMode_TypeDef;

// FLASH fixed programming time
typedef enum {
    FLASH_PROGRAMTIME_STANDARD = (uint8_t)0x00, // Standard programming time fixed at 1/2 tprog
    FLASH_PROGRAMTIME_TPROG    = (uint8_t)0x01  // Programming time fixed at tprog
} FLASH_ProgramTime_TypeDef;

//----------------------------------------------------------//
//                                         //
//----------------------------------------------------------//

void FLASH_DeInit(void);                                    //      
void FLASH_Unlock(void);                                    //     
void FLASH_Lock(void);                                      //     
void FLASH_ITConfig(FunctionalState NewState);              //  /   
void FLASH_EraseByte(uint32_t Address);                     //     
void FLASH_ProgramByte(uint32_t Address, uint8_t Data);     //     
void FLASH_ProgramWord(uint32_t Address, uint32_t Data);    //   4  WORD  
uint8_t FLASH_ReadByte(uint32_t Address);                   //     
void FLASH_SetProgrammingMode(FLASH_ProgramMode_TypeDef FLASH_ProgMode); //   ./.  
void FLASH_SetProgrammingTime(FLASH_ProgramTime_TypeDef FLASH_ProgTime); //     
void EEPROM_Unlock(void);                                   //     EEPROM
void EEPROM_Lock(void);                                     //     EEPROM
void EEPROM_EraseByte(uint32_t Address);                    //     EEPROM
void EEPROM_ProgramByte(uint32_t Address, uint8_t Data);    //     EEPROM
void EEPROM_ProgramWord(uint32_t Address, uint32_t Data);   //   4  WORD  EEPROM
uint8_t EEPROM_ReadByte(uint32_t Address);                  //     EEPROM

#endif
