//----------------------------------------------------------//
//       DS18B20     //
//----------------------------------------------------------//

//----------------------------------------------------------//
//                     //
//----------------------------------------------------------//

#include "DS18B20.h"
#include "main.h"

//----------------------------------------------------------//
//                            //
//----------------------------------------------------------//

#define DS18B20_DQ_High        (DS18B20_PORT_OUT |=  DS18B20_DQ_PIN)
#define DS18B20_DQ_Low         (DS18B20_PORT_OUT &=~ DS18B20_DQ_PIN)
#define DS18B20_DQ_In          (DS18B20_PORT_IN & DS18B20_DQ_PIN)

//----------------------------------------------------------//
//                   //
//----------------------------------------------------------//

uint8_t TemperatureSign = 0;

// ---------------------------------------------------------//
//  																				//
// ---------------------------------------------------------//

static void DS18B20_IO_In(void);                            //    DQ  
static void DS18B20_IO_Out(void);                           //    DQ  
static void DS18B20_Rst(void);                              //   
static uint8_t DS18B20_Check(void);                         //   /
static uint8_t DS18B20_ReadBit(void);                       //   1 
static uint8_t DS18B20_ReadByte(void);                      //   1 
static void DS18B20_WriteByte(uint8_t dat);                 //   1 
static void DS18B20_Start(void);                            //   

//----------------------------------------------------------//
//    DQ                      //
//----------------------------------------------------------//

static void DS18B20_IO_In(void)
{
  PD_DDR &=~ DS18B20_DQ_PIN;                                // PD2 -  
  PD_CR1 &=~ DS18B20_DQ_PIN;                                // PD2 -  ()
  PD_CR2 &=~ DS18B20_DQ_PIN;                                // EXTI - 
}

//----------------------------------------------------------//
//    DQ                     //
//----------------------------------------------------------//

static void DS18B20_IO_Out(void)
{
  PD_DDR |= DS18B20_DQ_PIN;                                 // PD2 -  
  PD_CR1 |= DS18B20_DQ_PIN;                                 // PD2 - -
  PD_CR2 |= DS18B20_DQ_PIN;                                 // PD2 - 10
}

//----------------------------------------------------------//
//                                     //
//----------------------------------------------------------//

static void DS18B20_Rst(void)  
{
  DS18B20_IO_Out();
  DS18B20_DQ_Low;
  delay_mks(750);
  DS18B20_DQ_High;
  delay_mks(15);
}

//----------------------------------------------------------//
//   /                 //
//----------------------------------------------------------//

static uint8_t DS18B20_Check(void)   
{
  uint8_t retry = 0;

  DS18B20_IO_In();
  while (DS18B20_DQ_In && retry < 100)
  {
    retry++;
    delay_mks(2);
  }
  if (retry >= 100) {
      return 1;
	}	else {
      retry = 0;
  }
  while (!DS18B20_DQ_In && retry < 120)
  {
		retry++;
    delay_mks(2);
  }
  if (retry >= 120) {
      return 1;
  }
  return 0;
}

//----------------------------------------------------------//
//   1                                     //
//----------------------------------------------------------//

static uint8_t DS18B20_ReadBit(void)
{
  uint8_t dat;

  //  
  //disableInterrupts();
  DS18B20_IO_Out();
  DS18B20_DQ_Low;
  delay_mks(2);
  DS18B20_DQ_High;
  DS18B20_IO_In();
  delay_mks(12);
  if (DS18B20_DQ_In) {
      dat = 1;
	} else {
      dat = 0;
  }
  delay_mks(50);
	//  
  //enableInterrupts();

  return dat;
}

//----------------------------------------------------------//
//   1                                    //
//----------------------------------------------------------//

static uint8_t DS18B20_ReadByte(void)
{
  uint8_t i = 0, j = 0;
  uint8_t dat = 0;

  //  1 
  for (i = 1; i <= 8; i++)
  {
    j = DS18B20_ReadBit();
    dat = (j << 7) | (dat >> 1);
  }

  return dat;
}

//----------------------------------------------------------//
//   1                                  //
//----------------------------------------------------------//

static void DS18B20_WriteByte(uint8_t dat)
{             
  uint8_t j = 0;
  uint8_t testb = 0;

  //  
  //disableInterrupts();
  DS18B20_IO_Out();
  //  1 
  for (j = 1; j <= 8; j++)
  {
    testb = dat & 0x01;
    dat = dat >> 1;
    if (testb) {
        DS18B20_DQ_Low;
        delay_mks(2);
        DS18B20_DQ_High;
        delay_mks(60);
    } else {
        DS18B20_DQ_Low;
        delay_mks(60);
        DS18B20_DQ_High;
        delay_mks(2);
    }
  }
  //  
  //enableInterrupts();
}
 
//----------------------------------------------------------//
//                                 //
//----------------------------------------------------------//

static void DS18B20_Start(void)
{
  DS18B20_Rst();           //     
  DS18B20_Check();         //      
  DS18B20_WriteByte(0xcc); //      
  DS18B20_WriteByte(0x44); //    
}

//----------------------------------------------------------//
//                                      //
//----------------------------------------------------------//

uint8_t DS18B20_Init(void)
{
  DS18B20_IO_Out();
  DS18B20_DQ_High;
  DS18B20_Rst();

  return DS18B20_Check();
}

//----------------------------------------------------------//
//               //
//----------------------------------------------------------//

short DS18B20_GetTemp(void)
{
  uint8_t TH = 0,TL = 0;   //      
  short temp = 0;          //      	

  DS18B20_Start();         // 
  //delay_ms(1000);
  DS18B20_Rst();           //   
  DS18B20_Check();         //    
  DS18B20_WriteByte(0xcc); //     
  DS18B20_WriteByte(0xbe); //   
  TL = DS18B20_ReadByte(); //      
  TH = DS18B20_ReadByte(); //      
  if (TH > 7) {
      TH=~TH;              // 
      TL=~TL;
      TemperatureSign = 0; //    
  } else {
      TemperatureSign = 1; // , 
  }
  temp = TH;
  temp <<= 8;
  temp += TL;
  temp = (float)temp * 0.625;
  if (TemperatureSign) {
      return temp;
  } else {
      return -temp;
  }
}
