//----------------------------------------------------------//
//     RTC DS3231                       //
//----------------------------------------------------------//

//----------------------------------------------------------//
//                     //
//----------------------------------------------------------//

#include "STM8S103.h"
#include "DS3231.h"
#include "main.h"

//----------------------------------------------------------//
//                            //
//----------------------------------------------------------//

#define I2C_SCL_PIN                 GPIO_PIN_4              // PB4 - C SCL
#define I2C_SDA_PIN                 GPIO_PIN_5              // PB5 -  SDA

#define I2C_OWN_ADDRESS7            0x00                    //   I2C

#define DS3231_ADDR                 0xD0                    // 1101 0000 

#define DS3231_ADDR_SECONDS         0x00
#define DS3231_ADDR_MINUTES         0x01
#define DS3231_ADDR_HOURS           0x02
#define DS3231_ADDR_DAY             0x03
#define DS3231_ADDR_DATE            0x04
#define DS3231_ADDR_MONTH           0x05
#define DS3231_ADDR_YEAR            0x06
#define DS3231_ADDR_A1_SECONDS      0x07
#define DS3231_ADDR_A1_MINUTES      0x08
#define DS3231_ADDR_A1_HOURS        0x09
#define DS3231_ADDR_A1_DAY_DATE     0x0A
#define DS3231_ADDR_A2_MINUTES      0x0B
#define DS3231_ADDR_A2_HOURS        0x0C
#define DS3231_ADDR_A2_DAY_DATE     0x0D
#define DS3231_ADDR_CONTROL         0x0E
#define DS3231_ADDR_STATUS          0x0F
#define DS3231_ADDR_AGING_OFFSET    0x10
#define DS3231_ADDR_TEMP_MSB        0x11
#define DS3231_ADDR_TEMP_LSB        0x12

//----------------------------------------------------------//
//                   //
//----------------------------------------------------------//

//----------------------------------------------------------//
//                                         //
//----------------------------------------------------------//

static void I2C_GPIO_Init(void);                            //   GPIO I2C
static void I2C_Init(void);                                 //   I2C
static void I2C_WriteData(uint8_t slave_addr, uint8_t addr, uint8_t * buffer, uint8_t size); // 
static void I2C_ReadData(uint8_t slave_addr, uint8_t addr, uint8_t * buffer, uint8_t size); // 
static void DS3231_WriteData(uint8_t addr, uint8_t * buffer, uint8_t size); // 
static void DS3231_ReadData(uint8_t addr, uint8_t * buffer, uint8_t size);// 
static uint8_t DS3231_Dec2BCD(uint8_t value);               //   Dec --> BCD
static uint8_t DS3231_BCD2Dec(uint8_t value);               //   BCD --> Dec

//----------------------------------------------------------//
//   GPIO I2C                           //
//----------------------------------------------------------//

static void I2C_GPIO_Init(void)
{
  PB_DDR |=  I2C_SCL_PIN | I2C_SDA_PIN;                     //   PB4 (SCL) & PB5 (SDA)
  PB_CR1 &=~ I2C_SCL_PIN | I2C_SDA_PIN;                     //  
  PB_CR2 |=  I2C_SCL_PIN | I2C_SDA_PIN;                     // 10
  PB_ODR |=  I2C_SCL_PIN | I2C_SDA_PIN;                     // .1
}

//----------------------------------------------------------//
//   I2C   100       //
//----------------------------------------------------------//

static void I2C_Init(void)
{
  uint8_t tmpccrh = 0;
  uint16_t result = 0x0004;

  I2C_CR2 |= 0x80; // SWRST
  I2C_CR2 &=~ 0x80;

  I2C_CR1 = I2C_CR1_RESET_VALUE;
  I2C_CR2 = I2C_CR2_RESET_VALUE;
  I2C_FREQR = I2C_FREQR_RESET_VALUE;
  I2C_OARL = I2C_OARL_RESET_VALUE;
  I2C_OARH = I2C_OARH_RESET_VALUE;
  I2C_ITR = I2C_ITR_RESET_VALUE;
  I2C_CCRL = I2C_CCRL_RESET_VALUE;
  I2C_CCRH = I2C_CCRH_RESET_VALUE;
  I2C_TRISER = I2C_TRISER_RESET_VALUE;

  I2C_FREQR &= (uint8_t)(~0x3F);
  I2C_FREQR |= 2; // 2MHz
  I2C_CR1 &= (uint8_t)(~I2C_CR1_PE);
  I2C_CCRH &= (uint8_t)(~(0x80 | 0x40 | 0x0F));
  I2C_CCRL &= (uint8_t)(~0xFF);
  result = (uint16_t)((2 * 1000000) / (100000 << (uint8_t)1)); // Calculate standard mode speed
  if (result < (uint16_t)0x0004) result = (uint16_t)0x0004;
  I2C_TRISER = (uint8_t)(2 + (uint8_t)1);
  I2C_CCRL = (uint8_t)result;
  I2C_CCRH = (uint8_t)((uint8_t)((uint8_t)(result >> 8) & 0x0F) | tmpccrh);
  I2C_CR1 |= (uint8_t)I2C_CR1_PE;
  I2C_CR2 &= (uint8_t)(~I2C_CR2_ACK);
  I2C_OARL = (uint8_t)(0x00);
  I2C_OARH = (uint8_t)((uint8_t)(0x00 | 0x40) | (uint8_t)((0x00 & (uint16_t)0x0300) >> (uint8_t)7));
}

//----------------------------------------------------------//
//   DS3231                             //
//----------------------------------------------------------//

void DS3231_Init(void)
{
  uint8_t Temp[1];

  I2C_GPIO_Init();                                          //  GPIO I2C
  I2C_Init();                                               //  I2C
  Temp[0] = 0x00;
  DS3231_WriteData(DS3231_ADDR_CONTROL, Temp, 1);
  DS3231_WriteData(DS3231_ADDR_STATUS, Temp, 1);
}

//----------------------------------------------------------//
//     I2C                           //
//----------------------------------------------------------//

static void I2C_WriteData(uint8_t slave_addr, uint8_t addr, uint8_t * buffer, uint8_t size)
{
  volatile uint8_t reg;
  uint8_t index = 0;

  I2C_CR1 |= I2C_CR1_PE;                                    //  I2C
  while(I2C_SR3 & I2C_SR3_BUSY);                            //   
  I2C_CR2 |= I2C_CR2_START;                                 //  
  while(!(I2C_SR1 & I2C_SR1_SB));                           //   
  reg = I2C_SR1;                                            //     
  slave_addr &= 0xFE;                                       //  0    TX/RX
  I2C_DR = slave_addr | I2C_DIRECTION_TX;                   //    + 
  while(!(I2C_SR1 & I2C_SR1_ADDR));                         //   
  reg = I2C_SR1;                                            //     
  reg = I2C_SR3;
  I2C_DR = addr;                                            //      
  while(!(I2C_SR1 & I2C_SR1_TXE));                          //    

  while(size)
  {
    size--;
    I2C_DR = buffer[index];
    index++;
    while(!(I2C_SR1 & I2C_SR1_TXE));
  }

  while(!(I2C_SR1 & I2C_SR1_BTF));
  I2C_CR2 |= I2C_CR2_STOP;                                  //  
  while((I2C_SR3 & I2C_SR3_MSL));
  I2C_CR1 &=~ I2C_CR1_PE;                                   //  I2C
}

//----------------------------------------------------------//
//     I2C                         //
//----------------------------------------------------------//

static void I2C_ReadData(uint8_t slave_addr, uint8_t addr, uint8_t * buffer, uint8_t size)
{
  volatile uint8_t reg;
  uint8_t index = 0;

  I2C_CR1 |= I2C_CR1_PE;                                    //  I2C
  while(I2C_SR3 & I2C_SR3_BUSY);                            //   
  I2C_CR2 |= I2C_CR2_START;                                 //  
  while(!(I2C_SR1 & I2C_SR1_SB));                           //   
  reg = I2C_SR1;                                            //     
  slave_addr &= 0xFE;                                       //  0    TX/RX
  I2C_DR = slave_addr | I2C_DIRECTION_TX;                   //    +  
  while(!(I2C_SR1 & I2C_SR1_ADDR));                         //   
  reg = I2C_SR1;                                            //     
  reg = I2C_SR3;
  I2C_DR = addr;                                            //   
  while(!(I2C_SR1 & I2C_SR1_TXE));
  while(!(I2C_SR1 & I2C_SR1_BTF));
  I2C_CR2 |= I2C_CR2_ACK;                                   //   
  I2C_CR2 |= I2C_CR2_START;                                 //  
  while(!(I2C_SR1 & I2C_SR1_SB));                           //   
  reg = I2C_SR1;                                            //     
  slave_addr &= 0xFE;                                       //  0    TX/RX
  I2C_DR = slave_addr | I2C_DIRECTION_RX;                   //    + 
  while(!(I2C_SR1 & I2C_SR1_ADDR));                         //   
  reg = I2C_SR1;                                            //     
  reg = I2C_SR3;

  while(size)
  {
    size--;
    if (size == 0) {
        I2C_CR2 &=~ I2C_CR2_ACK;
    }
    while(!(I2C_SR1 & I2C_SR1_RXNE));                       //   
    buffer[index] = I2C_DR;                                 //    
    index++;
  }

  I2C_CR2 |= I2C_CR2_STOP;                                  //  
  while((I2C_SR3 & I2C_SR3_MSL));
  I2C_CR1 &=~ I2C_CR1_PE;                                   //  I2C
}

//----------------------------------------------------------//
//     DS3231                        //
//----------------------------------------------------------//

static void DS3231_WriteData(uint8_t addr, uint8_t * buffer, uint8_t size)
{
  I2C_WriteData(DS3231_ADDR, addr, buffer, size);
}

//----------------------------------------------------------//
//     DS3231                      //
//----------------------------------------------------------//

static void DS3231_ReadData(uint8_t addr, uint8_t * buffer, uint8_t size)
{
  I2C_ReadData(DS3231_ADDR, addr, buffer, size);
}

//----------------------------------------------------------//
//      DS3231                  //
//----------------------------------------------------------//

void DS3231_SetDateTime(struct DS3231_DateTime *DS3231_Date)
{
  uint8_t Buff[7];

  Buff[0] = DS3231_Dec2BCD(DS3231_Date->Seconds);           // 
  Buff[1] = DS3231_Dec2BCD(DS3231_Date->Minutes);           // 
  Buff[2] = DS3231_Dec2BCD(DS3231_Date->Hour);              // 
  Buff[3] = DS3231_Dec2BCD(DS3231_Date->Day);               //  
  Buff[4] = DS3231_Dec2BCD(DS3231_Date->Date);              // 
  Buff[5] = DS3231_Dec2BCD(DS3231_Date->Month);             // 
  Buff[6] = DS3231_Dec2BCD(DS3231_Date->Year);              // 

  I2C_WriteData(DS3231_ADDR, DS3231_ADDR_SECONDS, Buff, 7);
}

//----------------------------------------------------------//
//      DS3231                 //
//----------------------------------------------------------//

void DS3231_GetDateTime(struct DS3231_DateTime *DS3231_Date)
{
  uint8_t Buff[7];

  I2C_ReadData(DS3231_ADDR, DS3231_ADDR_SECONDS, Buff, 7);

  DS3231_Date->Seconds = DS3231_BCD2Dec(Buff[0]);           // C
  DS3231_Date->Minutes = DS3231_BCD2Dec(Buff[1]);           // 
  DS3231_Date->Hour    = DS3231_BCD2Dec(Buff[2]);           // 
  DS3231_Date->Day     = DS3231_BCD2Dec(Buff[3]);           //  
  DS3231_Date->Date    = DS3231_BCD2Dec(Buff[4]);           // 
  DS3231_Date->Month   = DS3231_BCD2Dec(Buff[5]);           // 
  DS3231_Date->Year    = DS3231_BCD2Dec(Buff[6]);           // 
}

/*
//----------------------------------------------------------//
//    #1 DS3231                   //
//----------------------------------------------------------//

void DS3231_SetAlarm1(struct DS3231_Alarm1 *DS3231_WriteAlarm1)
{
  uint8_t Buff[4];

  Buff[0] = DS3231_Dec2BCD(DS3231_WriteAlarm1->Seconds);    // 
  Buff[1] = DS3231_Dec2BCD(DS3231_WriteAlarm1->Minutes);    // 
  Buff[2] = DS3231_Dec2BCD(DS3231_WriteAlarm1->Hour);       // 
  Buff[3] = DS3231_Dec2BCD(DS3231_WriteAlarm1->DY_DT);      //  
  //Buff[4] = DS3231_Dec2BCD(DS3231_WriteAlarm1->Date);       // 

  I2C_WriteData(DS3231_ADDR, DS3231_ADDR_A1_SECONDS, Buff, 4);
}

//----------------------------------------------------------//
//     #1 DS3231         //
//----------------------------------------------------------//

void DS3231_GetAlarm1(struct DS3231_Alarm1 *DS3231_ReadAlarm1)
{
  uint8_t Buff[4];

  I2C_ReadData(DS3231_ADDR, DS3231_ADDR_A1_SECONDS, Buff, 4);

  DS3231_ReadAlarm1->Seconds = DS3231_BCD2Dec(Buff[0] & 0x7f); // C
  DS3231_ReadAlarm1->Minutes = DS3231_BCD2Dec(Buff[1] & 0x7f); // 
  DS3231_ReadAlarm1->Hour    = DS3231_BCD2Dec(Buff[2] & 0x3f); // 
  DS3231_ReadAlarm1->DY_DT   = DS3231_BCD2Dec(Buff[3] & 0x3f); //  
  //DS3231_ReadAlarm1->Date    = DS3231_BCD2Dec(Buff[4] & 0x7f); // 
}

//----------------------------------------------------------//
//    #2 DS3231                   //
//----------------------------------------------------------//

void DS3231_SetAlarm2(struct DS3231_Alarm2 *DS3231_WriteAlarm2)
{
  uint8_t Buff[3];

  Buff[0] = DS3231_Dec2BCD(DS3231_WriteAlarm2->Minutes);    // 
  Buff[1] = DS3231_Dec2BCD(DS3231_WriteAlarm2->Hour);       // 
  Buff[2] = DS3231_Dec2BCD(DS3231_WriteAlarm2->DY_DT);      //  
  //Buff[3] = DS3231_Dec2BCD(DS3231_WriteAlarm2->Date);       // 

  I2C_WriteData(DS3231_ADDR, DS3231_ADDR_A2_MINUTES, Buff, 3);
}

//----------------------------------------------------------//
//     #2 DS3231         //
//----------------------------------------------------------//

void DS3231_GetAlarm2(struct DS3231_Alarm2 *DS3231_ReadAlarm2)
{
  uint8_t Buff[3];

  I2C_ReadData(DS3231_ADDR, DS3231_ADDR_A2_MINUTES, Buff, 3);

  DS3231_ReadAlarm2->Minutes = DS3231_BCD2Dec(Buff[0] & 0x7f); // 
  DS3231_ReadAlarm2->Hour    = DS3231_BCD2Dec(Buff[1] & 0x3f); // 
  DS3231_ReadAlarm2->DY_DT   = DS3231_BCD2Dec(Buff[2] & 0x3f); //  
  //DS3231_ReadAlarm2->Date    = DS3231_BCD2Dec(Buff[3] & 0x7f); // 
}
*/

//----------------------------------------------------------//
//   Dec --> BCD                          //
//----------------------------------------------------------//

static uint8_t DS3231_Dec2BCD(uint8_t value)
{
  return ((value / 10) << 4) + (value % 10);                // Convert Decimal to BCD
}

//----------------------------------------------------------//
//   BCD --> Dec                          //
//----------------------------------------------------------//

static uint8_t DS3231_BCD2Dec(uint8_t value)
{
  return ((value >> 4) * 10) + (value & 0x0f);              // Convert BCD to Decimal
}
