//----------------------------------------------------------//
//      SN74HC595              //
//----------------------------------------------------------//

//----------------------------------------------------------//
//                     //
//----------------------------------------------------------//

#include "main.h"
#include "stm8s_spi.h"
#include "SN74HC595.h"

//----------------------------------------------------------//
//                  //
//----------------------------------------------------------//

#define ST_PIN        GPIO_PIN_3
#define ST_PORT       PD_ODR

#define ST_HIGH()     ST_PORT |= ST_PIN;
#define ST_LOW()      ST_PORT &=~ ST_PIN;

//----------------------------------------------------------//
//                   //
//----------------------------------------------------------//

uint8_t IdxEsEff = 0;                                       //     . 

const uint8_t Led1ES[15][2] = {
  {0x00,0x01},{0x00,0x03},{0x00,0x07},{0x00,0x0f},
  {0x00,0x1f},{0x00,0x3f},{0x00,0x7f},{0x00,0xff},
  {0x80,0xff},{0xc0,0xff},{0xe0,0xff},{0xf0,0xff},
  {0xf8,0xff},{0xfc,0xff},{0x00,0x00}
};

const uint8_t Led2ES[9][2] = {
  {0x00,0x01},{0x04,0x03},{0x0c,0x07},{0x1c,0x0f},
  {0x3c,0x1f},{0x7c,0x3f},{0xfc,0x7f},{0xfc,0xff},{0x00,0x00}
};

extern uint16_t HeartBrightPWM;                             //    

//----------------------------------------------------------//
//                                         //
//----------------------------------------------------------//

extern void TIM2_SetLevelPWM(uint16_t PWM_Level);           //     

//----------------------------------------------------------//
//                                             //
//----------------------------------------------------------//

void SN74HC595_Init(void)
{
  // PD3 /   / - /  10 
  PD_DDR |= ST_PIN;
  PD_CR1 |= ST_PIN;
  PD_CR2 |= ST_PIN;
  ST_HIGH();
}

//----------------------------------------------------------//
//                                  //
//----------------------------------------------------------//

void Heart_Show(uint8_t *Buf)
{
  uint8_t i;

  for (i = 0; i < LED_REG_NUM; i++)
  {
    SPI_SendData(Buf[i]);
  }
  ST_LOW();
  ST_HIGH();
}

//----------------------------------------------------------//
// /    SN74HC595    //
//----------------------------------------------------------//

void Heart_State(uint8_t State)
{
  uint8_t i, buf[LED_REG_NUM];

  if (State != OFF) {
    for (i = 0; i < LED_REG_NUM; i++)
      buf[i] = 0xFF;
  } else {
    for (i = 0; i < LED_REG_NUM; i++)
      buf[i] = 0x00;
  }
  Heart_Show(&buf[0]);
}

//----------------------------------------------------------//
//    2-  c           //
//----------------------------------------------------------//

void Heart_Eff1Show(void)
{
  uint8_t i, y1, y2;
  uint8_t buf[LED_REG_NUM] = {0};

  buf[1] |= 0x01; // VD2
  Heart_Show(&buf[0]);
  delay_ms(100);
  y1 = 0x04;
  y2 = 0x02;
  for (i = 0; i < 6; i++)
  {
    buf[0] |= y1; //   
    buf[1] |= y2; //   
    Heart_Show(&buf[0]);
    y1 = y1 << 1;
    y2 = y2 << 1;
    delay_ms(100);
  }
  buf[1] |= 0x80; // VD9
  Heart_Show(&buf[0]);
  delay_ms(100);
}

//----------------------------------------------------------//
//            //
//----------------------------------------------------------//

void Heart_Eff2Show(void)
{
  uint8_t i, blink = 0;
  uint8_t buf[LED_REG_NUM];

  for (i = 0; i < 10; i++)
  {
    blink =! blink;
    if (blink) {
      buf[0] = 0x00;
      buf[1] = 0xff; //   
    } else {
      buf[0] = 0xff; //   
      buf[1] = 0x81;
    }
    Heart_Show(&buf[0]);
    delay_ms(100);
  }
}

//----------------------------------------------------------//
//                //
//----------------------------------------------------------//

void Heart_Eff3Show(void)
{
  uint8_t i, y1, y2;
  uint8_t buf[LED_REG_NUM] = {0};

  y1 = 0x04;
  y2 = 0x02;
  for (i = 0; i < 8; i++)
  {
    buf[0] = 0xff;
    buf[1] = 0xff;
    Heart_Show(&buf[0]);
    delay_ms(60);
    if (i == 0) {
      buf[1] &=~ 0x01; // VD2 
      Heart_Show(&buf[0]);
    } else {
      buf[0] &=~ y1; //   
      buf[1] &=~ y2; //   
      Heart_Show(&buf[0]);
      y1 = y1 << 1;
      y2 = y2 << 1;
    }
    delay_ms(60);
  }
}

//----------------------------------------------------------//
//                             //
//----------------------------------------------------------//

void Heart_Eff4Show(void)
{
  uint8_t i, y1, y2;
  uint8_t buf[LED_REG_NUM] = {0};	

  y1 = 0x80;
  y2 = 0x01;
  for (i = 0; i < 7; i++)
  {
    if (i == 0) {
      buf[1] |= y1 + y2; //  VD2 & VD9
    } else {
      buf[0] |= y1; //   
      buf[1] |= y2; //   
    }
    Heart_Show(&buf[0]);
    if (i != 0) y1 = y1 >> 1;
    y2 = y2 << 1;
    delay_ms(130);
  }
}

//----------------------------------------------------------//
//    (4  )            //
//----------------------------------------------------------//

void Heart_Eff5Show(void)
{
  uint8_t i, y1, y2;
  uint8_t buf[LED_REG_NUM] = {0xff, 0xff};	

  y1 = 0x02;
  y2 = 0x80;
  for (i = 0; i < 7; i++)
  {
    if (i == 0) {
      buf[1] &=~ 0x81; //  VD2 & VD9
    } else {
      buf[0] &=~ y1; //   
      buf[1] &=~ y2; //   
    }
    Heart_Show(&buf[0]);
    y1 = y1 << 1;
    y2 = y2 >> 1;
    delay_ms(130);
  }
}

//----------------------------------------------------------//
//    /                 //
//----------------------------------------------------------//

void Heart_Eff6Show(uint8_t Direction)
{
  uint8_t i, y1, y2;
  uint8_t buf[LED_REG_NUM];

  if (Direction == RIGHT) {
    y1 = 0x80;
    y2 = 0x01;
  } else {
    y1 = 0x02;
    y2 = 0x80;
  }
  for (i = 0; i < 7; i++)
  {
    buf[0] = 0xff;
    buf[1] = 0xff;
    if (i == 0) {
      buf[1] &=~ 0x81; //  VD2 & VD9
    } else {
      buf[0] &=~ y1; //   
      buf[1] &=~ y2; //   
    }
    Heart_Show(&buf[0]);
    if (Direction == RIGHT) {
      if (i != 0) y1 = y1 >> 1;
      y2 = y2 << 1;
    } else {
      y1 = y1 << 1;
      y2 = y2 >> 1;
    }
    delay_ms(100);
  }
}

//----------------------------------------------------------//
//  7  .                    //
//----------------------------------------------------------//

void Heart_Eff7Show(void)
{
  uint8_t i, j;
  const uint8_t led[4][2] = { {0x00,0x81},{0x84,0x42},{0x48,0x24},{0x30,0x18} };

  for (i = 0; i < 3; i++)
  {
    for (j = 0; j < 4; j++)
    {
      Heart_Show(&led[j][0]);
      delay_ms(100);
    }
  }
}

//----------------------------------------------------------//
//  8  .        //
//----------------------------------------------------------//

void Heart_Eff8Show(void)
{
  uint8_t i;
  const uint8_t led[4][2] = { {0x00,0x81},{0x84,0xc3},{0xcc,0xe7},{0xfc,0xff} };

  for (i = 0; i < 4; i++)
  {
    Heart_Show(&led[i][0]);
    delay_ms(250);
  }
}

//----------------------------------------------------------//
// ***********      ********** //
//----------------------------------------------------------//

//----------------------------------------------------------//
//  1  /                    //
//----------------------------------------------------------//

void Heart_Eff1EverySecShow(void)
{
  static uint8_t blink = 0;
  uint16_t i;

  blink =! blink;
  if (blink) {
    for (i = 50; i < HeartBrightPWM; i++)
    {
      TIM2_SetLevelPWM(i);
      delay_ms(1);
    }
  } else {
    for (i = HeartBrightPWM; i != 50; i--)
    {
      TIM2_SetLevelPWM(i);
      delay_ms(1);
    }
  }
}

//----------------------------------------------------------//
//  2                  //
//----------------------------------------------------------//

void Heart_Eff2EverySecShow(void)
{
  if (IdxEsEff >= 15) IdxEsEff = 0;
  Heart_Show(&Led1ES[IdxEsEff++][0]);
}

//----------------------------------------------------------//
//  3     2-   //
//----------------------------------------------------------//

void Heart_Eff3EverySecShow(void)
{
  if (IdxEsEff >= 9) IdxEsEff = 0;
  Heart_Show(&Led2ES[IdxEsEff++][0]);
}
