//----------------------------------------------------------//
//      TM1628                 //
//----------------------------------------------------------//

//----------------------------------------------------------//
//                     //
//----------------------------------------------------------//

#include "STM8S103.h"
#include "main.h"
#include "stm8s_spi.h"
#include "TM1628.h"

//----------------------------------------------------------//
//                            //
//----------------------------------------------------------//

#define TM1628_CS_PORT    PC_ODR                            // PC4 - C CS
#define TM1628_CS_PIN     GPIO_PIN_4                        // PC4 - C CS

#define CS_1    (TM1628_CS_PORT |= TM1628_CS_PIN)
#define CS_0    (TM1628_CS_PORT &=~ TM1628_CS_PIN)

//----------------------------------------------------------//
//                   //
//----------------------------------------------------------//

/*
**************************************************************
* LED Segments:         a
*                     ----
*                   f|    |b
*                    |  g |
*                     ----
*                   e|    |c
*                    |    |
*                     ----  o dp
*                       d
*   Register bits:
*      bit:  7  6  5  4  3  2  1  0
*           dp  g  f  e  d  c  b  a
**************************************************************
*/

//    K
const uint8_t TM1628_Font[] = {
  0x3F, // 0
  0x06, // 1
  0x5B, // 2
  0x4F, // 3
  0x66, // 4
  0x6D, // 5
  0x7D, // 6
  0x07, // 7
  0x7F, // 8
  0x6F  // 9
};

extern uint8_t TemperatureSign;                             //  

//----------------------------------------------------------//
//                                         //
//----------------------------------------------------------//

static void TM1628_WriteCmd(uint8_t Cmd);                   //     1628

//----------------------------------------------------------//
//                                      //
//----------------------------------------------------------//

void TM1628_Init(void)
{
  PC_DDR |= TM1628_CS_PIN;                                  // PC4 /   / - /  10
  PC_CR1 |= TM1628_CS_PIN;
  PC_CR2 |= TM1628_CS_PIN;

  TM1628_WriteCmd(0x03);                                    //  7  11 
  TM1628_WriteCmd(DISPLAY_ON);                              //     
  TM1628_DisplayClear();                                    //  
  TM1628_WriteCmd(ADDR_AUTO);                               //  .    
}

//----------------------------------------------------------//
//                                     //
//----------------------------------------------------------//

void TM1628_DisplayClear(void)
{
  uint8_t DispBuff[4];

  DispBuff[0] = _empty;
  DispBuff[1] = _empty;
  DispBuff[2] = _empty;
  DispBuff[3] = _empty;
  TM1628_DisplayArray(DispBuff);
}

//----------------------------------------------------------//
//                            //
//----------------------------------------------------------//
// 0 -  
// 7 -  

void TM1628_DisplaySetIntensity(uint8_t PWM)
{
  if (PWM > 7) PWM = 7;                                     //       .
  TM1628_WriteCmd(DISPLAY_ON | PWM);                        //     
}

//----------------------------------------------------------//
//                                   //
//----------------------------------------------------------//

void TM1628_DisplayOff(void)
{
  TM1628_WriteCmd(DISPLAY_OFF);
}

//----------------------------------------------------------//
//                                    //
//----------------------------------------------------------//

void TM1628_DisplayOn(void)
{
  TM1628_WriteCmd(DISPLAY_ON);
}

//----------------------------------------------------------//
//                                     //
//----------------------------------------------------------//

static void TM1628_WriteCmd(uint8_t Cmd)
{
  CS_0;
  SPI_SendData(Cmd);
  CS_1;
}

//----------------------------------------------------------//
//                     //
//----------------------------------------------------------//

void TM1628_WriteData(uint8_t Dig, uint8_t Data)
{
  TM1628_WriteCmd(ADDR_FIXED);
  CS_0;
  SPI_SendData(Dig);
  SPI_SendData(Data);
  CS_1;
}

//----------------------------------------------------------//
//                                    //
//----------------------------------------------------------//

void TM1628_DisplayArray(uint8_t SendData[])
{
  uint8_t i, DispBuff[8];

  DispBuff[0] = SendData[0];
  DispBuff[1] = 0;
  DispBuff[2] = SendData[1];
  DispBuff[3] = 0;
  DispBuff[4] = SendData[2];
  DispBuff[5] = 0;
  DispBuff[6] = SendData[3];
  TM1628_WriteCmd(ADDR_AUTO);
  CS_0;
  SPI_SendData(DIG1);
  for (i = 0; i < 7; i++)
  {
    SPI_SendData(DispBuff[i]);
  }
  CS_1;
}

//----------------------------------------------------------//
//     RTC DS3231                 //
//----------------------------------------------------------//

void TM1628_DisplayShowTime(uint8_t Hour, uint8_t Min)
{
  static uint8_t Blink = 0;
  uint8_t DispBuff[4] = {0};

  if (Hour >= 10) DispBuff[0] = TM1628_Font[((Hour%100)/10)];
  DispBuff[1] = TM1628_Font[(Hour%100)%10];
  DispBuff[2] = TM1628_Font[(Min/10)];
  DispBuff[3] = TM1628_Font[(Min%10)];
  Blink =! Blink;
  if (Blink) DispBuff[1] |= _dp;
  TM1628_DisplayArray(DispBuff);
}

//----------------------------------------------------------//
//     DS18B20                //
//----------------------------------------------------------//

void TM1628_DisplayShowTemp(short Temp)
{
  uint8_t TempBuff[4] = {0};
  uint8_t DispBuff[4] = {0};

  if (Temp < 0) Temp =- Temp;
  TempBuff[0] = (unsigned int)Temp/1000%10;
  TempBuff[1] = (unsigned int)Temp/100%10;
  TempBuff[2] = (unsigned int)Temp/10%10;
  TempBuff[3] = (unsigned int)Temp%10;
  if (Temp < 100) {
    if (!TemperatureSign) DispBuff[0] = _minus;
    DispBuff[1] = TM1628_Font[TempBuff[2]] | _dp;
    DispBuff[2] = TM1628_Font[TempBuff[3]];
    DispBuff[3] = _degree;
  } else if (Temp >= 100 && Temp < 1000) {
    if (TemperatureSign) {
      DispBuff[0] = TM1628_Font[TempBuff[1]];
      DispBuff[1] = TM1628_Font[TempBuff[2]] | _dp;
      DispBuff[2] = TM1628_Font[TempBuff[3]];
      DispBuff[3] = _degree;
    } else {
      DispBuff[0] = _minus;
      DispBuff[1] = TM1628_Font[TempBuff[1]];
      DispBuff[2] = TM1628_Font[TempBuff[2]] | _dp;
      DispBuff[3] = TM1628_Font[TempBuff[3]];
    }
  } else if (Temp >= 1000 && Temp <= 1250) {
    DispBuff[0] = TM1628_Font[TempBuff[0]];
    DispBuff[1] = TM1628_Font[TempBuff[1]];
    DispBuff[2] = TM1628_Font[TempBuff[2]];
    DispBuff[3] = _degree;
  }
  TM1628_DisplayArray(DispBuff);
}

//----------------------------------------------------------//
//                                      //
//----------------------------------------------------------//

void TM1628_RunningString(uint8_t DispData[], uint8_t Amount, uint16_t DelayMs)
{
  uint8_t i;
  uint8_t DispBuff[4];
  uint8_t SegData[64]; //    4       

  for (i = 0; i < 4; i++)
  { //   4  
    SegData[i] = 0x00;
  }
  for (i = 0; i < Amount; i++)
  { //       (  )
    SegData[i + 4] = DispData[i];
  }
  for (i = Amount + 4; i < Amount + 8; i++)
  { //   4    
    SegData[i] = 0x00;
  }
  for (i = 0; i <= Amount + 4; i++)
  { // 
    DispBuff[0] = SegData[i];
    DispBuff[1] = SegData[i+1];
    DispBuff[2] = SegData[i+2];
    DispBuff[3] = SegData[i+3];
    TM1628_DisplayArray(DispBuff);
    delay_ms(DelayMs);
  }
}
