//----------------------------------------------------------//
//                                        //
//----------------------------------------------------------//

//----------------------------------------------------------//
//                     //
//----------------------------------------------------------//

#include "STM8S103.h"
#include "stm8s_flash.h"

//----------------------------------------------------------//
//             //
//----------------------------------------------------------//

void FLASH_DeInit(void)
{
  uint8_t Temp;

  FLASH_CR1 = 0x00;
  FLASH_CR2 = 0x00;
  FLASH_NCR2 = 0xFF;
  FLASH_IAPSR &=~ 0x08;
  FLASH_IAPSR &=~ 0x02;
  Temp = FLASH_IAPSR;
}

//----------------------------------------------------------//
//                          //
//----------------------------------------------------------//

void FLASH_Unlock(void)
{
  FLASH_PUKR = 0x56;
  FLASH_PUKR = 0xAE;
  while(!(FLASH_IAPSR & FLASH_FLAG_PUL));
}

//----------------------------------------------------------//
//                             //
//----------------------------------------------------------//

void FLASH_Lock(void)
{
  FLASH_IAPSR &= 0xFD;
}

//----------------------------------------------------------//
//  /               //
//----------------------------------------------------------//

void FLASH_ITConfig(FunctionalState NewState)
{
  if (NewState != DISABLE) {
      FLASH_CR1 |= 0x02;
  } else {
      FLASH_CR1 &=~ 0x02;
  }
}

//----------------------------------------------------------//
//                                 //
//----------------------------------------------------------//

void FLASH_EraseByte(uint32_t Address)
{
  *(@near uint8_t*) (uint16_t)Address = FLASH_CLEAR_BYTE;
  while(!(FLASH_IAPSR & (FLASH_FLAG_EOP | FLASH_FLAG_WR_PG_DIS)));
}

//----------------------------------------------------------//
//                                  //
//----------------------------------------------------------//

void FLASH_ProgramByte(uint32_t Address, uint8_t Data)
{
  *(@near uint8_t*) (uint16_t)Address = Data;
  while(!(FLASH_IAPSR & (FLASH_FLAG_EOP | FLASH_FLAG_WR_PG_DIS)));
}

//----------------------------------------------------------//
//                              //
//----------------------------------------------------------//

uint8_t FLASH_ReadByte(uint32_t Address)
{
  return (*(@near uint8_t*) (uint16_t)Address);
}

//----------------------------------------------------------//
//   4  WORD                        //
//----------------------------------------------------------//

void FLASH_ProgramWord(uint32_t Address, uint32_t Data)
{
  FLASH_CR2 |= FLASH_CR2_WPRG;
  FLASH_NCR2 &=~ FLASH_NCR2_NWPRG;
  *((@near uint8_t*)(uint16_t)Address)       = *((uint8_t*)(&Data));
  *(((@near uint8_t*)(uint16_t)Address) + 1) = *((uint8_t*)(&Data)+1);
  *(((@near uint8_t*)(uint16_t)Address) + 2) = *((uint8_t*)(&Data)+2);
  *(((@near uint8_t*)(uint16_t)Address) + 3) = *((uint8_t*)(&Data)+3);
  while(!(FLASH_IAPSR & (FLASH_FLAG_EOP | FLASH_FLAG_WR_PG_DIS)));
}

//----------------------------------------------------------//
//   ./.       //
//----------------------------------------------------------//

void FLASH_SetProgrammingMode(FLASH_ProgramMode_TypeDef FLASH_ProgMode)
{
  if (FLASH_ProgMode == FLASH_PROGRAMMODE_STANDARD) {
      FLASH_CR2 |= 0x01;
      FLASH_NCR2 &=~ 0x01;
  } else {
      FLASH_CR2 |= 0x10;
      FLASH_NCR2 &=~ 0x10;
  }
}

//----------------------------------------------------------//
//                //
//----------------------------------------------------------//

void FLASH_SetProgrammingTime(FLASH_ProgramTime_TypeDef FLASH_ProgTime)
{
  FLASH_CR1 &=~ 0x01;
  FLASH_CR1 |= FLASH_ProgTime;
}

//----------------------------------------------------------//
//     EEPROM                    //
//----------------------------------------------------------//

void EEPROM_Unlock(void)
{
  FLASH_DUKR = 0xAE;
  FLASH_DUKR = 0x56;
  while(!(FLASH_IAPSR & FLASH_FLAG_DUL));
}

//----------------------------------------------------------//
//     EEPROM                       //
//----------------------------------------------------------//

void EEPROM_Lock(void)
{
  FLASH_IAPSR &= 0xF7;
}

//----------------------------------------------------------//
//     EEPROM                           //
//----------------------------------------------------------//

void EEPROM_EraseByte(uint32_t Address)
{
  *(@near uint8_t*) (uint16_t)Address = FLASH_CLEAR_BYTE;
  while(!(FLASH_IAPSR & (FLASH_FLAG_EOP | FLASH_FLAG_WR_PG_DIS)));
}

//----------------------------------------------------------//
//     EEPROM                            //
//----------------------------------------------------------//

void EEPROM_ProgramByte(uint32_t Address, uint8_t Data)
{
  *(@near uint8_t*) (uint16_t)Address = Data;
  while(!(FLASH_IAPSR & (FLASH_FLAG_EOP | FLASH_FLAG_WR_PG_DIS)));
}

//----------------------------------------------------------//
//     EEPROM                       //
//----------------------------------------------------------//

uint8_t EEPROM_ReadByte(uint32_t Address)
{
  return (*(@near uint8_t*) (uint16_t)Address); 
}

//----------------------------------------------------------//
//   4  WORD  EEPROM                     //
//----------------------------------------------------------//

void EEPROM_ProgramWord(uint32_t Address, uint32_t Data)
{
  FLASH_CR2 |= FLASH_CR2_WPRG;
  FLASH_NCR2 &=~ FLASH_NCR2_NWPRG;
  *((@near uint8_t*)(uint16_t)Address)       = *((uint8_t*)(&Data));
  *(((@near uint8_t*)(uint16_t)Address) + 1) = *((uint8_t*)(&Data)+1);
  *(((@near uint8_t*)(uint16_t)Address) + 2) = *((uint8_t*)(&Data)+2);
  *(((@near uint8_t*)(uint16_t)Address) + 3) = *((uint8_t*)(&Data)+3);
  while(!(FLASH_IAPSR & (FLASH_FLAG_EOP | FLASH_FLAG_WR_PG_DIS)));
}
