//----------------------------------------------------------//
// :   Love Clock v1.0                                //
// : -  /      //
// :  4  7  0.56                //
// :   14   5                     //
//                                                      //
//  /                                          //
//   /                     //
//                                        //
//                                    //
// :  STM8S103F3P6, TM1628, SN74HC595, DS18B20         //
// :     ST Visual Develop v4.3.12 & Cosmic               //
// : 16.08.21                            //
// :  27.08.21 /                    //
// :                                         //
// :  ivan.kz2014@gmail.com                            //
// :   https://domdevice.com                            //
// :  domdevice                                        //
//----------------------------------------------------------//

//----------------------------------------------------------//
//                     //
//----------------------------------------------------------//

#include "STM8S103.h"
#include "stm8s_flash.h"
#include "stm8s_spi.h"
#include "main.h"
#include "menu.h"
#include "TM1628.h"
#include "SN74HC595.h"
#include "DS3231.h"
#include "DS18B20.h"

//----------------------------------------------------------//
//                                    //
//----------------------------------------------------------//

#define TIM2_PERIOD      10000                              //   100

//----------------------------------------------------------//
//  ,       //
//----------------------------------------------------------//

struct DS3231_DateTime DS3231_ReadDate;                     //     

// DomDevice
const uint8_t DomDeviceTxt[11] = { _d,_O,_ML,_MR,_empty,_d,_E,_U,_I,_C,_E };

//   
const uint8_t DefaultTxt[7] = { _d,_E,_F,_A,_U,_L,_t };

extern uint8_t IdxEsEff;                                    //     . 
volatile uint8_t Flag1Sec = 1;                              //  1 
short Temperature = 0;                                      // 
uint8_t Mode = DAY;                                         //  . 0 - , 1 - 
uint8_t KeyProcessing = 0;                                  //  . 1 - , 0 - 
uint8_t CheckSetups = 0;                                    // 0 -   , 1 - 
uint8_t HourNightOn = 22;                                   //    
uint8_t HourNightOff = 8;                                   //    
uint8_t NightMode = 1;                                      //  . 0 -  , 1 - 2 
uint8_t TimeShowClock = 10;                                 //     ,   10
uint8_t TimeShowTemp = 3;                                   //     ,   3
uint8_t Brightness = 2;                                     //   
uint8_t HeartEff = 4;                                       //   . . 1 - 3 .
uint8_t HeartEffSec = 1;                                    // .  . 0 - , 1 - 9 
uint8_t HeartEffSecPer = 1;                                 //  .  . 0 - , 1 - 60.
uint8_t HeartBrightDayPer = 10;                             //     %. 0 - 
uint16_t HeartBrightPWM = 0;                                //    

//----------------------------------------------------------//
//                                         //
//----------------------------------------------------------//

void CLK_Init(void);                                        //    HSI 16MHz
void GPIO_Init(void);                                       //   GPIO
void TIM2_Init(void);                                       //    2   
void TIM4_Init(void);                                       //    4
void TIM2_SetLevelPWM(uint16_t PWM_Level);                  //     
void Hello(void);                                           //  
void Love(void);                                            //    Love
void GoToReboot(void);                                      //  
void ResetToDefault(void);                                  //     
void EE_SavingConfig(void);                                 //     EEPROM
void Heart_SetBrightness(uint8_t Value);                    //      

main(void)
{
  volatile uint8_t CntSec = 0;                              //  
  volatile uint8_t HeartCntMin = 0;                         //     
  volatile uint8_t Flag1Min = 0;                            //  1 
  uint8_t HeartEffSecondary = 0;                            //    
  uint8_t ShowItem[2] = {0};                                //   
  uint8_t SkipEff = 0;                                      //    .   .
  uint16_t i = 0;                                           //   

  for (i = 0; i < 0xffff; i++) { }
  CLK_Init();                                               //    HSI 16MHz
  GPIO_Init();                                              //  GPIO
  TIM2_Init();                                              //   2   
  TIM4_Init();                                              //   4
  SPI_Init();                                               //  SPI
  SN74HC595_Init();                                         //  74HC595
  Heart_State(OFF);                                         //  
  TM1628_Init();                                            //  1628
  FLASH_DeInit();                                           //  FLASH & EEPROM
  DS3231_Init();                                            //  RTC DS3231

  //    
  if ((KEY_PORT2 & (KEY_PLUS | KEY_MINUS)) == 0) {
    delay_ms(2000);
    if ((KEY_PORT2 & (KEY_PLUS | KEY_MINUS)) == 0) ResetToDefault();
  }

  //  
  CheckSetups = EEPROM_ReadByte(EE_ADDR_CHECK_SETUPS);
  if (CheckSetups == 1) {
    HourNightOn = EEPROM_ReadByte(EE_ADDR_HOUR_NIGHT_ON);
    HourNightOff = EEPROM_ReadByte(EE_ADDR_HOUR_NIGHT_OFF);
    NightMode = EEPROM_ReadByte(EE_ADDR_NIGHT_MODE);
    TimeShowClock = EEPROM_ReadByte(EE_ADDR_TIME_SHOW_CLOCK);
    TimeShowTemp = EEPROM_ReadByte(EE_ADDR_TIME_SHOW_TEMP);
    Brightness = EEPROM_ReadByte(EE_ADDR_BRIGHTNESS);
    HeartEff = EEPROM_ReadByte(EE_ADDR_HEART_EFF);
    HeartEffSec = EEPROM_ReadByte(EE_ADDR_HEART_EFF_SEC);
    HeartEffSecPer = EEPROM_ReadByte(EE_ADDR_HEART_EFF_SEC_PER);
    HeartBrightDayPer = EEPROM_ReadByte(EE_ADDR_HEART_BRIGH_D_PNT);
  } else ResetToDefault();                                  //   
  TM1628_DisplaySetIntensity(Brightness);                   //   
  Heart_SetBrightness(HeartBrightDayPer);                   //   
  HeartBrightPWM = (uint16_t)(((uint32_t)HeartBrightDayPer * 10000) / 100); //   PWM  %  

  //   
  if ((KEY_PORT1 & KEY_MENU) == 0) {
    delay_ms(2000);
    if ((KEY_PORT1 & KEY_MENU) == 0) Menu_Processing();
  }

  //      DS18B20?
  if (DS18B20_Init() == 0) {
    Temperature = DS18B20_GetTemp();                        //   (-   85)
  } else {
    TimeShowTemp = 0;
  }
  ShowItem[0] = TimeShowClock;                              //   
  ShowItem[1] = (ShowItem[0] + TimeShowTemp);               //  + 
  Hello();                                                  // 
  if (!HeartEff) Heart_State(OFF);                          //   .  . 
  enableInterrupts();                                       //   

  while(1) {
    if (Flag1Sec == 1) {
      //    
      DS3231_GetDateTime(&DS3231_ReadDate);
      if (DS3231_ReadDate.Seconds == 0) Flag1Min = 1;

      // ,    ?
      if (HourNightOn != HourNightOff) {
        if (DS3231_ReadDate.Hour == HourNightOn && Mode != NIGHT) {
          if (NightMode == 0 || NightMode == 2) {           //    
            if (HeartEff) {
              Heart_State(OFF);                             //     --> 
              Heart_SetBrightness(0);                       //  PWM
            }
            if (NightMode == 0) TM1628_DisplayOff();        //  
            else TM1628_DisplaySetIntensity(INTENSITY_MIN); //     
          } else {                                          //     
            if (HeartEff) {
              Heart_State(ON);                              //     --> 
              TIM2_SetLevelPWM(10);                         //      0.1%
            }
            TM1628_DisplaySetIntensity(INTENSITY_MIN);      //     
          }
          CntSec = 0;                                       //     
          Mode = NIGHT;                                     //     
        } else {
          if (DS3231_ReadDate.Hour == HourNightOff && Mode != DAY) {
            if (HeartEff) Heart_State(ON);                  //   .  . 
            Heart_SetBrightness(HeartBrightDayPer);         //     
            TM1628_DisplaySetIntensity(Brightness);         //     
            HeartCntMin = 0;                                //     
            Flag1Min = 0;                                   //   1 .
            CntSec = 0;                                     //     
            Mode = DAY;                                     //     
          }
        }
      }

      //   /  /  
      if (Mode == DAY || (Mode == NIGHT && NightMode != 0)) {
        CntSec++;
        if (CntSec > ShowItem[1]) CntSec = 0;
        if (CntSec <= ShowItem[0]) {                        //  
          TM1628_DisplayShowTime(DS3231_ReadDate.Hour, DS3231_ReadDate.Minutes);
        } else {
          if (TimeShowTemp!= 0 && CntSec <= ShowItem[1]) {  //  
            if (CntSec == (ShowItem[0] + 1)) {
              Temperature = DS18B20_GetTemp();
              TM1628_DisplayShowTemp(Temperature);
            }
          }
        }
      }

      //      
      if (Mode != NIGHT && HeartBrightDayPer != 0) {
        if (Flag1Min == 1) {
          if (HeartEffSecPer != 0) {
            if (++HeartCntMin == HeartEffSecPer) {
              Heart_SetBrightness(HeartBrightDayPer);
              if (HeartEffSec != 0) HeartEffSecondary = HeartEffSec; //   . 
              else HeartEffSecondary++;  //   
              switch (HeartEffSecondary) {
                case 1:
                  Heart_Eff1Show();      //  #1   2-  c 
                  break;
                case 2:
                  Heart_Eff2Show();      //  #2      
                  break;
                case 3:
                  Heart_Eff3Show();      //  #3       
                  break;
                case 4:
                  Heart_Eff4Show();      //  #4    
                  break;
                case 5:
                  Heart_Eff5Show();      //  #5   (4  )
                  break;
                case 6:
                  Heart_Eff6Show(RIGHT); //  #6    
                  break;
                case 7:
                  Heart_Eff6Show(LEFT);  //  #6    
                  break;
                case 8:
                  Heart_Eff7Show();      //  #7  .   
                  break;
                case 9:
                  Heart_Eff8Show();      //  #8  .     
                  HeartEffSecondary = 0;
                  break;
                default:
                  break;
              }
              SkipEff = 1;
              IdxEsEff = 0;
              HeartCntMin = 0;
              if (HeartEff) Heart_State(ON); //      
              else Heart_State(OFF);
            }
          }
          Flag1Min = 0;
        }

        //      
        if (HeartEff > 1 && !SkipEff) {
          switch (HeartEff) {
            case 2:
              Heart_Eff1EverySecShow(); //  #1  / ( )
              break;
            case 3:
              Heart_Eff2EverySecShow(); //  #2   
              break;
            case 4:
              Heart_Eff3EverySecShow(); //  #3     2- 
              break;
            default:
              break;
          }
        }
        if (SkipEff) SkipEff = 0;
      }

      //          
      if (((KEY_PORT1 & KEY_MENU) == 0) || ((KEY_PORT1 & KEY_OK) == 0) ||
          ((KEY_PORT2 & KEY_PLUS) == 0) || ((KEY_PORT2 & KEY_MINUS) == 0)) {
        if (Mode == NIGHT && NightMode == 0) {
          TM1628_DisplaySetIntensity(INTENSITY_MIN);
          for (i = 0; i < 5; i++)
          {
            TM1628_DisplayShowTime(DS3231_ReadDate.Hour, DS3231_ReadDate.Minutes);
            delay_ms(1000);
          }
          Temperature = DS18B20_GetTemp();
          TM1628_DisplayShowTemp(Temperature);
          delay_ms(5000);
          TM1628_DisplayClear();
          TM1628_DisplayOff();
        }
      }
      Flag1Sec = 0;
    }
  }
}

//----------------------------------------------------------//
//     HSI 16MHz          //
//----------------------------------------------------------//

void CLK_Init(void)
{
  CLK_ICKR = 0;                                             // Reset the Internal Clock Register.
  CLK_ICKR |= 0x01;                                         // Enable the HSI.
  CLK_ECKR = 0;                                             // Disable the external clock.
  while ((CLK_ICKR & 0x02) == 0);                           // Wait for the HSI to be ready for use.
  CLK_CKDIVR = 0;                                           // Ensure the clocks are running at full speed.
  CLK_PCKENR1 = 0xff;                                       // Enable all peripheral clocks.
  CLK_PCKENR2 = 0xff;                                       // Ditto.
  CLK_CCOR = 0;                                             // Turn off CCO.
  CLK_HSITRIMR = 0;                                         // Turn off any HSIU trimming.
  CLK_SWIMCCR = 0;                                          // Set SWIM to run at clock / 2.
  CLK_SWR = 0xe1;                                           // Use HSI as the clock source.
  CLK_SWCR = 0;                                             // Reset the clock switch control register.
  CLK_SWCR |= 0x02;                                         // Enable switching.
  while ((CLK_SWCR & 0x01) != 0);                           // Pause while the clock switch is busy.
}

//----------------------------------------------------------//
//   GPIO                               //
//----------------------------------------------------------//

void GPIO_Init(void)
{
  // PA3 / PWM.   / - /  10
  PA_DDR |= PWM_PIN;
  PA_CR1 |= PWM_PIN;
  PA_CR2 |= PWM_PIN;

  // PC3 / TLI.  SQW 1  RTC DS3231 /   /    / EXTI  
  PC_DDR &=~ 0x08;
  PC_CR1 |= 0x08;
  PC_CR2 |= 0x08;

  // PA1, PA2, PD5, PD6 /   /    / EXTI  
  PA_DDR &=~ KEY_PLUS | KEY_MINUS;
  PA_CR1 |= KEY_PLUS | KEY_MINUS;
  PA_CR2 &=~ KEY_PLUS | KEY_MINUS;

  PD_DDR &=~ KEY_MENU | KEY_OK;
  PD_CR1 |= KEY_MENU | KEY_OK;
  PD_CR2 &=~ KEY_MENU | KEY_OK;
}

//----------------------------------------------------------//
//    2                 //
//   TIM2 CH3 (PA3) (10)                        //
//----------------------------------------------------------//

void TIM2_Init(void)
{
  //      ARP
  //   ,    ,  
  TIM2_CR1 = 0x80;
  TIM2_CCER1 = 0x00;                                        //   1  2
  TIM2_CCER2 = 0x00;                                        //   3
  TIM2_PSCR = 0x04;                                         //   16 16000000/16 = 1
  TIM2_SR1 &=~ 0x03;                                        //   
  TIM2_SR2 &=~ 0x02;                                        //   
  TIM2_IER = 0x00;                                          //     2
  TIM2_ARRH = (uint8_t)(TIM2_PERIOD >> 8);                  //   0-10000.   1000000/10000 = 100
  TIM2_ARRL = (uint8_t)(TIM2_PERIOD);
  TIM2_CCR3H = 0x00;                                        //   PWM
  TIM2_CCR3L = 0x00;
  //  3   ,     
  //  PWM1 -       
  TIM2_CCMR3 = 0x68;
  // A   ,   3
  TIM2_CCER2 = 0x01;
  //    3,   3 
  //TIM2_EGR = 0x09;
  TIM2_CR1 |= 0x01;                                         //  
}

//----------------------------------------------------------//
//                 //
//   0 - 10000                               //
//----------------------------------------------------------//

void TIM2_SetLevelPWM(uint16_t PWM_Level)
{
  TIM2_CCR3H = (uint8_t)(PWM_Level >> 8);
  TIM2_CCR3L = (uint8_t)(PWM_Level);
}

//----------------------------------------------------------//
//                   //
//----------------------------------------------------------//

void Heart_SetBrightness(uint8_t Value)
{
  uint16_t PWM;

  PWM = (((uint32_t)Value * 10000) / 100);
  TIM2_SetLevelPWM(PWM);
}

//----------------------------------------------------------//
//    4                          //
//----------------------------------------------------------//

void TIM4_Init(void)
{
  TIM4_PSCR = 0x04;                                         //    16
  TIM4_EGR |= 0x01;                                         //   
  TIM4_CR1 = 0x01;                                          //    
}

//----------------------------------------------------------//
//       4                      //
//----------------------------------------------------------//

void delay_mks(uint16_t mks)
{
  while (mks > 0xFF)
  {
    TIM4_CNTR = 1;
    while (TIM4_CNTR);
    mks -= 0xFF;
  }
  TIM4_CNTR = 0;
  while (TIM4_CNTR < mks);
}

//----------------------------------------------------------//
//       4                       //
//----------------------------------------------------------//

void delay_ms(uint16_t ms)
{
  while (ms--) delay_mks(1000);
}

//----------------------------------------------------------//
//                                        //
//----------------------------------------------------------//

void Hello(void)
{
  TM1628_RunningString(DomDeviceTxt, sizeof(DomDeviceTxt), 200);
  Love();
  Heart_Eff1Show();
  delay_ms(1000);
}

//----------------------------------------------------------//
//    Love                             //
//----------------------------------------------------------//

void Love(void)
{
  uint8_t i, j;
  uint8_t buf[4] = {0};
  const uint8_t size[4] = { 3,6,5,5 };
  const uint8_t LoveSeg[4][6] = {
    { 0x20,0x10,0x08,0x00,0x00,0x00 },
    { 0x01,0x20,0x10,0x08,0x04,0x02 },
    { 0x20,0x10,0x08,0x04,0x02,0x00 },
    { 0x01,0x20,0x10,0x08,0x40,0x00 }
  };

  for (i = 0; i < 4; i++)
  {
    for (j = 0; j < size[i]; j++)
    {
      buf[i] |= LoveSeg[i][j];
      TM1628_DisplayArray(&buf[0]);
      delay_ms(50);
    }
  }
}

//----------------------------------------------------------//
//                                       //
//----------------------------------------------------------//

void GoToReboot(void)
{
  WWDG_CR |= 0x80; // 
  while (1) {}
}

//----------------------------------------------------------//
//                         //
//----------------------------------------------------------//

void ResetToDefault(void)
{
  EE_SavingConfig();
  TM1628_DisplaySetIntensity(Brightness);
  TM1628_RunningString(DefaultTxt, sizeof(DefaultTxt), 280);
}

//----------------------------------------------------------//
//     EEPROM                     //
//----------------------------------------------------------//

void EE_SavingConfig(void)
{
  CheckSetups = 1;
  FLASH_SetProgrammingTime(FLASH_PROGRAMTIME_STANDARD);
  EEPROM_Unlock();
  EEPROM_ProgramByte(EE_ADDR_CHECK_SETUPS, CheckSetups);
  EEPROM_ProgramByte(EE_ADDR_HOUR_NIGHT_ON, HourNightOn);
  EEPROM_ProgramByte(EE_ADDR_HOUR_NIGHT_OFF, HourNightOff);
  EEPROM_ProgramByte(EE_ADDR_NIGHT_MODE, NightMode);
  EEPROM_ProgramByte(EE_ADDR_TIME_SHOW_CLOCK, TimeShowClock);
  EEPROM_ProgramByte(EE_ADDR_TIME_SHOW_TEMP, TimeShowTemp);
  EEPROM_ProgramByte(EE_ADDR_BRIGHTNESS, Brightness);
  EEPROM_ProgramByte(EE_ADDR_HEART_EFF, HeartEff);
  EEPROM_ProgramByte(EE_ADDR_HEART_EFF_SEC, HeartEffSec);
  EEPROM_ProgramByte(EE_ADDR_HEART_EFF_SEC_PER, HeartEffSecPer);
  EEPROM_ProgramByte(EE_ADDR_HEART_BRIGH_D_PNT, HeartBrightDayPer);
  EEPROM_Lock();
}
